@@ -45,6 +45,7 @@ from ..utils.core import (
 
 from ..debugging import (log, debug_log)
 
+from itertools import islice
 import json
 import time
 import random
@@ -88,7 +89,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         {
             'name': 'Get chat messages from livestream',
             'params': {
-                'url': 'https://www.youtube.com/watch?v=5qap5aO4i9A',
+                'url': 'https://www.youtube.com/watch?v=jfKfPfyJRdk',
                 'timeout': 5
             }
         },
@@ -200,8 +201,22 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'messages_condition': lambda messages: len(messages) > 0,
             }
         },
+        {  # https://github.com/xenova/chat-downloader/issues/178#issuecomment-1330029347
+            'name': 'Chat replay with membership gifts',
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=cb0h-KbpDo8',
+                'start_time': '5:22:20',
+                'end_time': '5:22:35',
+                'message_groups': ['all']
+            },
+            'expected_result': {
+                'message_types': ['text_message', 'sponsorships_gift_purchase_announcement', 'ticker_sponsor_item'],
+                'action_types': ['add_chat_item', 'add_live_chat_ticker_item'],
+                'messages_condition': lambda messages: len(messages) > 0,
+            }
+        },
+
         {
-            # 874:24:05 current test
             'name': 'Get chat messages from an unplayable stream.',
             'params': {
                 'url': 'https://www.youtube.com/watch?v=V2Afni3S-ok',
@@ -214,6 +229,17 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 'messages_condition': lambda messages: len(messages) > 0,
             }
         },
+        {  # https://github.com/xenova/chat-downloader/issues/175#issue-1438381085
+            'name': 'Chat replay with a message that has no author name',
+            'params': {
+                'url': 'https://www.youtube.com/watch?v=-JU0rbfPECY',
+                'timeout': 5,
+                'start_time': '1:53:29',
+                'end_time': '1:53:30',
+            }
+        },
+
+
 
         # TESTING FOR ERRORS
         {
@@ -380,7 +406,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         {
             'name': 'Clip does not have a chat replay.',
             'params': {
-                'url': 'https://www.youtube.com/clip/UgxJiPo-4EeSYDfrYp94AaABCQ',
+                'url': 'https://www.youtube.com/clip/UgwVu73xQ5FUiGnteZJ4AaABCQ',
             },
             'expected_result': {
                 'error': NoChatReplay,
@@ -398,7 +424,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         {
             'name': "Clip not available. The clip can be unavailable if it was deleted, or if the video it's based on was removed or edited.",
             'params': {
-                'url': 'https://youtube.com/clip/UgwVu73xQ5FUiGnteZJ4AaABCQ',
+                'url': 'https://youtube.com/clip/UgxJiPo-4EeSYDfrYp94AaABCQ',
             },
             'expected_result': {
                 'error': VideoUnavailable,
@@ -415,7 +441,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         }
     ]
 
-    _YT_INITIAL_BOUNDARY_RE = r'\s*(?:var\s+meta|</script|\n)'
+    _YT_INITIAL_BOUNDARY_RE = r'\s*(?:var\s+(?:meta|head)|</script|\n)'
     _YT_INITIAL_DATA_RE = r'(?:window\s*\[\s*["\']ytInitialData["\']\s*\]|ytInitialData)\s*=\s*({.+?})\s*;' + \
         _YT_INITIAL_BOUNDARY_RE
     _YT_INITIAL_PLAYER_RESPONSE_RE = r'ytInitialPlayerResponse\s*=\s*({.+?})\s*;' + \
@@ -439,6 +465,9 @@ class YouTubeChatDownloader(BaseChatDownloader):
             'membership_item',
             'paid_message',
             'paid_sticker',
+
+            # Gifts
+            'sponsorships_gift_purchase_announcement',
         ],
         'tickers': [
             # superchat messages which appear ticker (at the top)
@@ -530,12 +559,12 @@ class YouTubeChatDownloader(BaseChatDownloader):
                         youtube(?:kids)?\.com
                     )/
                     (?:
-                        (?P<type>channel|c|user)/
+                        (?P<type>channel/|c/|user/|@)
                     )?
                     (?P<id>[a-zA-Z0-9_-]+)'''
     }
 
-    @ staticmethod
+    @staticmethod
     def _get_source_image_url(url):
         index = url.find('=')
         if index >= 0:
@@ -543,7 +572,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         else:
             return url
 
-    @ staticmethod
+    @staticmethod
     def _parse_youtube_link(text):
         if text.startswith(('/redirect', 'https://www.youtube.com/redirect')):  # is a redirect link
             info = dict(parse.parse_qsl(parse.urlsplit(text).query))
@@ -555,7 +584,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
         else:  # is a normal link
             return text
 
-    @ staticmethod
+    @staticmethod
     def _parse_navigation_endpoint(navigation_endpoint, default_text=''):
         try:
             return YouTubeChatDownloader._parse_youtube_link(
@@ -563,11 +592,11 @@ class YouTubeChatDownloader(BaseChatDownloader):
         except Exception:
             return default_text
 
-    @ staticmethod
+    @staticmethod
     def _parse_text(info):
         return YouTubeChatDownloader._parse_runs(info)['message'] or YouTubeChatDownloader._get_simple_text(info)
 
-    @ staticmethod
+    @staticmethod
     def _parse_runs(run_info, parse_links=True):
         """ Reads and parses YouTube formatted messages (i.e. runs). """
 
@@ -598,22 +627,21 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 emoji = run['emoji']
                 emoji_id = emoji.get('emojiId')
 
-                name = multi_get(emoji, 'shortcuts', 0)
+                name = multi_get(emoji, 'shortcuts', 0) or emoji_id
 
-                if name:
-                    if emoji_id and emoji_id not in message_emotes:
+                if emoji_id and emoji_id not in message_emotes:
 
-                        # TODO change to remapping?
-                        message_emotes[emoji_id] = {
-                            'id': emoji_id,
-                            'name': name,
-                            'shortcuts': emoji.get('shortcuts'),
-                            'search_terms': emoji.get('searchTerms'),
-                            'images': YouTubeChatDownloader._parse_thumbnails(emoji.get('image', {})),
-                            'is_custom_emoji': emoji.get('isCustomEmoji', False)
-                        }
+                    # TODO change to remapping?
+                    message_emotes[emoji_id] = {
+                        'id': emoji_id,
+                        'name': name,
+                        'shortcuts': emoji.get('shortcuts'),
+                        'search_terms': emoji.get('searchTerms'),
+                        'images': YouTubeChatDownloader._parse_thumbnails(emoji.get('image', {})),
+                        'is_custom_emoji': emoji.get('isCustomEmoji', False)
+                    }
 
-                    message_info['message'] += name
+                message_info['message'] += name
 
             else:
                 # unknown run
@@ -624,7 +652,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
         return message_info
 
-    @ staticmethod
+    @staticmethod
     def _parse_item(item, info=None, offset=0):
         if info is None:
             info = {}
@@ -657,8 +685,17 @@ class YouTubeChatDownloader(BaseChatDownloader):
                 info.update(YouTubeChatDownloader._parse_item(
                     renderer, offset=offset))
 
+        header = item_info.get('header')
+        if header:
+            info.update(YouTubeChatDownloader._parse_item(
+                header, offset=offset))
+
         BaseChatDownloader._move_to_dict(info, 'author')
 
+        # Sometimes YouTube channels can have no names, so, account for this
+        if 'author' in info and 'name' not in info['author']:
+            info['author']['name'] = ''
+
         # TODO determine if youtube glitch has occurred
         # round(time_in_seconds/timestamp) == 1
         time_in_seconds = info.get('time_in_seconds')
@@ -693,7 +730,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
         return info
 
-    @ staticmethod
+    @staticmethod
     def _parse_badges(badge_items):
         badges = []
 
@@ -733,7 +770,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
             # print(badges)
         return badges
 
-    @ staticmethod
+    @staticmethod
     def _parse_thumbnails(item):
 
         # sometimes thumbnails come as a list
@@ -753,7 +790,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
         return final
 
-    @ staticmethod
+    @staticmethod
     def _parse_action_button(item):
         endpoint = multi_get(item, 'buttonRenderer', 'navigationEndpoint')
 
@@ -762,7 +799,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
             'text': multi_get(item, 'buttonRenderer', 'text', 'simpleText') or ''
         }
 
-    @ staticmethod
+    @staticmethod
     def _get_simple_text(item):
         return item.get('simpleText')
 
@@ -792,7 +829,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
     # https://en.wikipedia.org/wiki/ISO_4217
     # e.g. 'CHF', 'COP', 'HUF', 'PLN', 'RUB', 'SEK', 'PEN', 'ARS', 'CLP', 'NOK', 'BAM', 'SGD'
 
-    @ staticmethod
+    @staticmethod
     def _parse_currency(item):
         mixed_text = item.get('simpleText') or str(item)
 
@@ -841,6 +878,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
         # ticker_sponsor_item
         'detailText': r(None, _parse_runs, True),
+        'detailIcon': r('detail_icon', lambda x: x.get('iconType')),
         'customThumbnail': r('badge_icons', _parse_thumbnails),
 
         # membership_item
@@ -876,12 +914,18 @@ class YouTubeChatDownloader(BaseChatDownloader):
         # tooltip
         'detailsText': r(None, _parse_runs, True),
 
+        # gifts
+        'primaryText': r('message', _parse_text),
+
+        'bannerProperties': 'banner_properties',
+        'headerOverlayImage': r('header_overlay_image', _parse_thumbnails),
     }
 
     _COLOUR_KEYS = [
         # paid_message
         'authorNameTextColor', 'timestampColor', 'bodyBackgroundColor',
         'headerTextColor', 'headerBackgroundColor', 'bodyTextColor',
+        'textInputBackgroundColor',
 
         # paid_sticker
         'backgroundColor', 'moneyChipTextColor', 'moneyChipBackgroundColor',
@@ -945,6 +989,12 @@ class YouTubeChatDownloader(BaseChatDownloader):
             'liveChatPaidStickerRenderer',
             'liveChatModeChangeMessageRenderer',  # e.g. slow mode enabled
 
+            # Gifting
+            'liveChatSponsorshipsGiftPurchaseAnnouncementRenderer',  # purchase
+            'liveChatSponsorshipsGiftRedemptionAnnouncementRenderer',  # receive
+
+            'liveChatSponsorshipsHeaderRenderer',
+
             # TODO find examples of:
             # 'liveChatPurchasedProductMessageRenderer',  # product purchased
             # liveChatLegacyPaidMessageRenderer
@@ -967,7 +1017,13 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
     # [message deleted] or [message retracted]
     _KNOWN_REMOVE_ACTION_TYPES = {
-        'markChatItemsByAuthorAsDeletedAction': [  # TODO ban?
+        'removeChatItemAction': [
+            'banUser',
+        ],
+        'removeChatItemByAuthorAction': [
+            'banUser',
+        ],
+        'markChatItemsByAuthorAsDeletedAction': [
             'banUser'  # deletedStateMessage
         ],
         'markChatItemAsDeletedAction': [
@@ -978,8 +1034,8 @@ class YouTubeChatDownloader(BaseChatDownloader):
     _KNOWN_ADD_BANNER_TYPES = {
         'addBannerToLiveChatCommand': [
             'liveChatBannerRenderer',
-            'liveChatBannerHeaderRenderer'
-            'liveChatTextMessageRenderer'
+            'liveChatBannerHeaderRenderer',
+            'liveChatTextMessageRenderer',
         ]
     }
 
@@ -1003,7 +1059,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
     # liveChatPaidMessageFooterRenderer
     # liveChatProductButtonRenderer
     # liveChatPurchaseMessageEndpoint
-    # removeChatItemAction
     # replaceLiveChatRendererAction
     # showLiveChatDialogAction
     # showLiveChatSurveyCommand
@@ -1067,8 +1122,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
         for item in items:
             yield self._YT_VIDEO_TEMPLATE.format(item['video_id'])
 
-        # downloader.get_playlist_items
-
     _LIVE_PLAYLIST_URL = _YT_HOME + '/channel/UC4R8DWoMoI7CAwX8_LjQHig'
 
     def _get_testing_items(self):
@@ -1085,19 +1138,12 @@ class YouTubeChatDownloader(BaseChatDownloader):
         for section in sections:
             section_info = section['itemSectionRenderer']['contents'][0]['shelfRenderer']
 
-            # print(section_info)
-
-            # section_title = section_info['title']['runs'][0]['text']
-            # print(section_title)
-
-            # items = section_info['content']['horizontalListRenderer']['items']
-
             playlist_url = self._YT_HOME + \
                 section_info['endpoint']['commandMetadata']['webCommandMetadata']['url']
 
             yield from self.get_playlist_items(playlist_url)
 
-    @ staticmethod
+    @staticmethod
     def _get_rendered_content(yt_info, tab_index=0):
         return yt_info['contents']['twoColumnBrowseResultsRenderer']['tabs'][tab_index]['tabRenderer']['content'][
             'sectionListRenderer']['contents'][0]['itemSectionRenderer']['contents'][0]
@@ -1105,24 +1151,38 @@ class YouTubeChatDownloader(BaseChatDownloader):
     _VIDEO_REMAPPING = {
         'videoId': 'video_id',
         'title': r('title', lambda x: YouTubeChatDownloader._parse_runs(x)['message']),
+        'videoType': 'video_type',
         'viewCountText': r('view_count', lambda x: YouTubeChatDownloader._parse_text(x)),
         'shortViewCountText': r('short_view_count', lambda x: YouTubeChatDownloader._parse_text(x)),
 
         # 'videoId', 'thumbnail', 'title', 'viewCountText', 'navigationEndpoint', 'ownerBadges', 'trackingParams', 'shortViewCountText', 'menu', 'thumbnailOverlays'
     }
 
-    @ staticmethod
+    @staticmethod
     def _parse_video(video_renderer):
+        # Get video type:
+        # One of DEFAULT, UPCOMING, LIVE
+        video_type = 'DEFAULT'
+        thumbnail_overlays = multi_get(
+            video_renderer, 'thumbnailOverlays') or []
+        for thumbnail_overlay in thumbnail_overlays:
+            video_type = multi_get(
+                thumbnail_overlay, 'thumbnailOverlayTimeStatusRenderer', 'style')
+            if video_type:
+                break
+
+        video_renderer['videoType'] = video_type
+
         return r.remap_dict(video_renderer, YouTubeChatDownloader._VIDEO_REMAPPING)
 
-    _VIDEO_STATUS_REMAPPING = {
-        'all': 'all',
-        'live': (501, 'Live now'),
-        'upcoming': (502, 'Upcoming live streams'),
-        'past': (503, 'Past live streams')
+    _VIDEO_TYPE_REMAPPING = {
+        # Name : url component
+        'videos': 'videos',
+        'shorts': 'shorts',
+        'live': 'streams',
     }
 
-    def get_user_videos(self, channel_id=None, user_id=None, custom_username=None, video_status='all', params=None):
+    def get_user_videos(self, channel_id=None, user_id=None, custom_username=None, handle=None, video_type='videos', params=None):
         """Retrieve all videos listed on the user's channel
 
         If more than one of `channel_id`, `user_id` and `custom_username`
@@ -1134,15 +1194,18 @@ class YouTubeChatDownloader(BaseChatDownloader):
         :param user_id: The user's ID, defaults to None
             (e.g., https://www.youtube.com/user/<user_id>)
         :type user_id: str, optional
-        :param custom_username: [description], defaults to None
+        :param custom_username: User's custom username, defaults to None
             (e.g., https://www.youtube.com/c/<custom_username>)
         :type custom_username: str, optional
-        :param video_status: Determines which videos will be retrieved, defaults to 'all'.
-            Must be one of 'all', 'live', 'upcoming' or 'past'.
-        :type video_status: str, optional
+        :param handle: User's handle, defaults to None
+            (e.g., https://www.youtube.com/@<handle>)
+        :type handle: str, optional
+        :param video_type: Determines which videos will be retrieved, defaults to 'videos'.
+            Must be one of 'videos', 'live', or 'shorts'.
+        :type video_type: str, optional
         :param params: Additional program parameters, defaults to None
         :type params: dict, optional
-        :raises ValueError: If no user is specified or an invalid video_status is specified
+        :raises ValueError: If no user is specified or an invalid video_type is specified
         :raises UserNotFound: If the user cannot be found
         :raises NoVideos: If the channel has no videos
         :yield: The next video
@@ -1153,50 +1216,52 @@ class YouTubeChatDownloader(BaseChatDownloader):
         _type = ''
         if channel_id:
             _id = channel_id
-            _type = 'channel'
+            _type = 'channel/'
         elif user_id:
             _id = user_id
-            _type = 'user'
+            _type = 'user/'
         elif custom_username:
             _id = custom_username
-            _type = 'c'
+            _type = 'c/'
+        elif handle:
+            _id = handle
+            _type = '@'
         else:
             raise ValueError('No user type specified.')
 
-        # live, past, upcoming
-        vid_type = self._VIDEO_STATUS_REMAPPING.get(video_status.lower())
+        video_type = video_type.lower()
+        vid_type = self._VIDEO_TYPE_REMAPPING.get(video_type)
 
         if not vid_type:
             raise ValueError(
-                f'Invalid argument passed for video_status. Must be one of {set(self._VIDEO_STATUS_REMAPPING.keys())}')
+                f'Invalid argument passed for video_type. Must be one of {set(self._VIDEO_TYPE_REMAPPING.keys())}')
 
-        user_url = f'https://www.youtube.com/{_type}/{_id}'
+        user_url = f'https://www.youtube.com/{_type}{_id}'
+        yt_info, ytcfg, _ = self._get_initial_info(
+            f'{user_url}/{vid_type}', params)
 
-        if vid_type == 'all':
-            vids_url = f'{user_url}/videos'
-        else:
-            vids_url = f'{user_url}/videos?view=2&live_view={vid_type[0]}'
-
-        yt_info, ytcfg, _ = self._get_initial_info(vids_url, params)
-
-        section_list_renderer = multi_get(
-            yt_info, 'contents', 'twoColumnBrowseResultsRenderer', 'tabs', 1, 'tabRenderer', 'content', 'sectionListRenderer')
-        if not section_list_renderer:
+        tabs = multi_get(yt_info, 'contents',
+                         'twoColumnBrowseResultsRenderer', 'tabs')
+        if not tabs:
             raise UserNotFound(f'Unable to find user: "{user_url}"')
 
-        sub_menu_items = multi_get(
-            section_list_renderer, 'subMenu', 'channelSubMenuRenderer', 'contentTypeSubMenuItems')
-        if not sub_menu_items:
-            raise NoVideos('This channel has no videos.')
+        page_contents = None
+        for tab in tabs:
+            tab_data = tab.get('tabRenderer', {})
+            if not tab_data or not tab_data.get('selected'):
+                continue
 
-        # Check that the returned grid is what was asked for
-        # YouTube tries to correct your mistake by selecting the uploads tab
-        # if you try to access a tab that is not visible.
-        selected = list(filter(lambda x: x['selected'], sub_menu_items))
-        if vid_type != 'all' and (not selected or selected[0]['title'] != vid_type[1]):
-            log('debug',
-                f'"{vid_type[1]}" tab is not visible for this channel (i.e. there are no such videos).')
-            return
+            tab_title = tab_data.get('title', '').lower()
+            # Check that the returned grid is what was asked for
+            # YouTube tries to correct your mistake by selecting the home tab
+            # if you try to access a tab that is not visible.
+            if tab_title != video_type.lower():
+                log('debug',
+                    f'"{tab_title}" tab is not visible for this channel (i.e. there are no such videos).')
+                raise NoVideos(
+                    f'This channel has no videos of the requested type ({video_type}).')
+
+            page_contents = tab_data.get('content')
 
         api_key = ytcfg.get('INNERTUBE_API_KEY')
         continuation_url = self._YOUTUBE_BROWSE_API_TEMPLATE.format(api_key)
@@ -1210,8 +1275,8 @@ class YouTubeChatDownloader(BaseChatDownloader):
         first_time = True
         while True:
             if first_time:
-                items = multi_get(section_list_renderer, 'contents', 0,
-                                  'itemSectionRenderer', 'contents', 0, 'gridRenderer', 'items')
+                items = multi_get(
+                    page_contents, 'richGridRenderer', 'contents')
                 first_time = False
             else:
                 continuation_params['continuation'] = continuation
@@ -1225,7 +1290,8 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
             continuation = None
             for item in items:
-                vid = item.get('gridVideoRenderer')
+                vid = multi_get(item, 'richItemRenderer',
+                                'content', 'videoRenderer')
                 continuation_item = item.get('continuationItemRenderer')
 
                 if vid:
@@ -1239,17 +1305,48 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
     def get_playlist_items(self, playlist_url, params=None):
 
-        yt_initial_data, ytcfg, player_response_info = self._get_initial_info(
+        yt_initial_data, ytcfg, _ = self._get_initial_info(
             playlist_url, params)
 
-        items = self._get_rendered_content(
-            yt_initial_data)['playlistVideoListRenderer']['contents']
+        page_contents = self._get_rendered_content(yt_initial_data)
 
-        for item in items:
-            playlist_video = item.get('playlistVideoRenderer')
+        # TODO remove code duplication
+        api_key = ytcfg.get('INNERTUBE_API_KEY')
+        continuation_url = self._YOUTUBE_BROWSE_API_TEMPLATE.format(api_key)
 
-            if playlist_video:
-                yield self._parse_video(playlist_video)
+        continuation_params = {
+            'context': ytcfg.get('INNERTUBE_CONTEXT') or {}
+        }
+        continuation = None
+        first_time = True
+        while True:
+            if first_time:
+                items = multi_get(
+                    page_contents, 'playlistVideoListRenderer', 'contents')
+                first_time = False
+            else:
+                continuation_params['continuation'] = continuation
+                yt_info = self._get_continuation_info(
+                    continuation_url, params, json=continuation_params)
+                items = multi_get(yt_info, 'onResponseReceivedActions',
+                                  0, 'appendContinuationItemsAction', 'continuationItems')
+
+            if not items:
+                break
+
+            continuation = None
+            for item in items:
+                vid = item.get('playlistVideoRenderer')
+                continuation_item = item.get('continuationItemRenderer')
+
+                if vid:
+                    yield self._parse_video(vid)
+                elif continuation_item:
+                    continuation = multi_get(
+                        continuation_item, 'continuationEndpoint', 'continuationCommand', 'token')
+
+            if not continuation:
+                break
 
     _CONSENT_ID_REGEX = r'PENDING\+(\d+)'
     # https://github.com/ytdl-org/youtube-dl/blob/a8035827177d6b59aca03bd717acb6a9bdd75ada/youtube_dl/extractor/youtube.py#L251
@@ -1492,6 +1589,8 @@ class YouTubeChatDownloader(BaseChatDownloader):
                     raise LoginRequired(error_message)
                 elif status == 'UNPLAYABLE':
                     raise VideoUnplayable(error_message)
+                elif status == 'LIVE_STREAM_OFFLINE':
+                    raise ChatDisabled(error_message)
                 else:
                     log('debug',
                         f'Unknown playability status: {status}. {playability_status}')
@@ -1586,7 +1685,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
         # Top chat replay - Some messages, such as potential spam, may not be visible
         # Live chat replay - All messages are visible
-        chat_type = params.get('chat_type').title()  # Live or Top
+        chat_type = params.get('chat_type', 'live').title()  # Live or Top
         continuation_index = 0 if chat_type == 'Top' else 1
         continuation_info = list(initial_continuation_info.items())[
             continuation_index]
@@ -1727,7 +1826,7 @@ class YouTubeChatDownloader(BaseChatDownloader):
                         original_item = action
                         if original_action_type == 'markChatItemAsDeletedAction':
                             original_message_type = 'deletedMessage'
-                        else:  # markChatItemsByAuthorAsDeletedAction
+                        else:  # markChatItemsByAuthorAsDeletedAction, removeChatItemAction
                             original_message_type = 'banUser'
 
                         data = self._parse_item(original_item, data, offset)
@@ -1735,7 +1834,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
                     elif original_action_type in self._KNOWN_REPLACE_ACTION_TYPES:
                         original_item = multi_get(
                             action, original_action_type, 'replacementItem')
-
                         original_message_type = try_get_first_key(
                             original_item)
                         data = self._parse_item(original_item, data, offset)
@@ -1743,7 +1841,6 @@ class YouTubeChatDownloader(BaseChatDownloader):
                     elif original_action_type in self._KNOWN_TOOLTIP_ACTION_TYPES:
                         original_item = multi_get(
                             action, original_action_type, 'tooltip')
-
                         original_message_type = try_get_first_key(
                             original_item)
                         data = self._parse_item(original_item, data, offset)
@@ -1755,21 +1852,13 @@ class YouTubeChatDownloader(BaseChatDownloader):
                         if original_item:
                             original_message_type = try_get_first_key(
                                 original_item)
-
-                            header = original_item[original_message_type].get(
-                                'header')
-                            parsed_header = self._parse_item(
-                                header, offset=offset)
-                            header_message = parsed_header.get('message')
-
                             contents = original_item[original_message_type].get(
                                 'contents')
                             parsed_contents = self._parse_item(
                                 contents, offset=offset)
 
-                            data.update(parsed_header)
                             data.update(parsed_contents)
-                            data['header_message'] = header_message
+
                         else:
                             debug_log(
                                 'No bannerRenderer item',
@@ -1973,16 +2062,23 @@ class YouTubeChatDownloader(BaseChatDownloader):
 
     def _get_chat_by_user(self, match, params):
         match_id = match.group('id')
-        user_type = match.group('type')  # channel|c|user
+        user_type = match.group('type') or ''
+        user_type = user_type.rstrip('/')  # channel|c|user|@|
 
         if user_type == 'channel':
             return self.get_chat_by_channel_id(match_id, params)
 
-        if user_type == 'user':
+        elif user_type == 'user':
             return self.get_chat_by_user_id(match_id, params)
 
-        # Otherwise assume custom username
-        return self.get_chat_by_custom_username(match_id, params)
+        elif user_type in ('c', ''):
+            return self.get_chat_by_custom_username(match_id, params)
+
+        elif user_type == '@':
+            return self.get_chat_by_handle(match_id, params)
+
+        else:
+            raise ValueError(f'Invalid user_type: {user_type}')
 
     def get_chat_by_channel_id(self, channel_id, params):
         return self._get_chat_by_user_args({
@@ -2005,6 +2101,11 @@ class YouTubeChatDownloader(BaseChatDownloader):
             'custom_username': custom_username
         }, params)
 
+    def get_chat_by_handle(self, handle, params):
+        return self._get_chat_by_user_args({
+            'handle': handle
+        }, params)
+
     def _get_chat_by_user_args(self, user_video_args, params):
         # TODO add param for wait time
         # params['exit_on_fail'] = True
@@ -2022,35 +2123,43 @@ class YouTubeChatDownloader(BaseChatDownloader):
         list_of_vids_to_ignore = params.get('ignore') or []
 
         sleep_amount = 30  # params.get('retry_timeout')
+        # For efficiency purposes, do not loop over all past broadcasts if not found
+        max_vids_to_try = 5
 
         while True:
-            for video_status in ('live', 'upcoming'):
-                # prioritise live videos
-                for video in self.get_user_videos(**user_video_args, video_status=video_status, params=params):
-                    video_id = video['video_id']
-                    video_title = video['title']
 
-                    if video_id in list_of_vids_to_ignore:
-                        log('debug', f'Skipping video with ID: "{video_id}"')
-                        continue
+            vids = self.get_user_videos(
+                **user_video_args, video_type='live', params=params)
 
-                    try:
-                        chat = self.get_chat_by_video_id(video_id, params)
+            for video in islice(vids, max_vids_to_try):
+                video_id = video['video_id']
 
-                        log('info',
-                            f"Found a{'n upcoming' if video_status == 'upcoming' else ''} livestream: \"{video_title}\" ({video_id}).")
+                if video['video_type'] not in ('LIVE', 'UPCOMING'):
+                    log('debug',
+                        f'Skipping video with ID: "{video_id}" (not live/upcoming)')
+                    continue
 
-                        for key, value in vars(chat).items():  # Update chat item
-                            if key != 'chat' and not key.startswith('_'):
-                                setattr(chat_item, key, value)
+                if video_id in list_of_vids_to_ignore:
+                    log('debug', f'Skipping video with ID: "{video_id}"')
+                    continue
 
-                        yield from chat
-                        break
+                try:
+                    chat = self.get_chat_by_video_id(video_id, params)
 
-                    except ChatDownloaderError as e:
-                        # For some reason, doesn't work
-                        log('warning',
-                            f"Unable to get chat for \"{video['title']}\" ({video_id}) due to an error: \"{e}\"")
+                    log('info',
+                        f"Found a livestream: \"{video['title']}\" ({video_id}).")
+
+                    for key, value in vars(chat).items():  # Update chat item
+                        if key != 'chat' and not key.startswith('_'):
+                            setattr(chat_item, key, value)
+
+                    yield from chat
+                    break
+
+                except ChatDownloaderError as e:
+                    # For some reason, doesn't work
+                    log('warning',
+                        f"Unable to get chat for \"{video['title']}\" ({video_id}) due to an error: \"{e}\"")
 
             log('info',
                 f'There are no active or upcoming livestreams with a live chat. Retrying in {sleep_amount} seconds.')
